﻿namespace Ecng.Trading.Xaml
{
	using System.Windows.Threading;

	using Ecng.Trading.Algo.Candles;
	using Ecng.Trading.BusinessEntities;
	using Ecng.Trading.Quik;

	/// <summary>
	/// Вспомогательный класс для синхронизованных объектов.
	/// </summary>
	public static class GuiObjectHelper
	{
		/// <summary>
		/// Создать синхронизованный шлюз <see cref="GuiTrader{T}"/>.
		/// </summary>
		/// <typeparam name="T">Тип шлюза, который необходимо синхронизовать. Например, <see cref="QuikTrader"/>.</typeparam>
		/// <param name="trader">Шлюз, который необходимо обернуть в <see cref="GuiTrader{T}"/>.</param>
		/// <returns>Cинхронизованный шлюз <see cref="GuiTrader{T}"/>.</returns>
		public static GuiTrader<T> GuiSyncTrader<T>(this T trader)
			where T : ITrader
		{
			return new GuiTrader<T>(trader);
		}

		/// <summary>
		/// Создать синхронизованный шлюз <see cref="GuiTrader{T}"/>.
		/// </summary>
		/// <typeparam name="T">Тип шлюза, который необходимо синхронизовать. Например, <see cref="QuikTrader"/>.</typeparam>
		/// <param name="trader">Шлюз, который необходимо обернуть в <see cref="GuiTrader{T}"/>.</param>
		/// <returns>Cинхронизованный шлюз <see cref="GuiTrader{T}"/>.</returns>
		/// <param name="dispatcher">Специальный класс для синхронизации с GUI потоком.</param>
		public static GuiTrader<T> GuiSyncTrader<T>(this T trader, Dispatcher dispatcher)
			where T : ITrader
		{
			return new GuiTrader<T>(trader, dispatcher);
		}

		/// <summary>
		/// Создать синхронизованный менеджер свечек <see cref="GuiCandleManager{T}"/>.
		/// </summary>
		/// <typeparam name="T">Тип менеджера свечек, который необходимо синхронизовать. Например, <see cref="CandleManager"/>.</typeparam>
		/// <param name="candleManager">Менеджер свечек, который необходимо обернуть в <see cref="GuiCandleManager{T}"/>.</param>
		/// <returns>Синхронизованный менеджер свечек <see cref="GuiCandleManager{T}"/>.</returns>
		public static GuiCandleManager<T> GuiSyncCandleManager<T>(this T candleManager)
			where T : ICandleManager
		{
			return new GuiCandleManager<T>(candleManager);
		}

		/// <summary>
		/// Создать синхронизованный менеджер свечек <see cref="GuiCandleManager{T}"/>.
		/// </summary>
		/// <typeparam name="T">Тип менеджера свечек, который необходимо синхронизовать. Например, <see cref="CandleManager"/>.</typeparam>
		/// <param name="candleManager">Менеджер свечек, который необходимо обернуть в <see cref="GuiCandleManager{T}"/>.</param>
		/// <returns>Синхронизованный менеджер свечек <see cref="GuiCandleManager{T}"/>.</returns>
		/// <param name="dispatcher">Специальный класс для синхронизации с GUI потоком.</param>
		public static GuiCandleManager<T> GuiSyncCandleManager<T>(this T candleManager, Dispatcher dispatcher)
			where T : ICandleManager
		{
			return new GuiCandleManager<T>(candleManager, dispatcher);
		}
	}
}