﻿namespace Ecng.Trading.Xaml
{
	using System;
	using System.Collections.ObjectModel;
	using System.Collections.Specialized;
	using System.Linq;

	using Ecng.Collections;
	using Ecng.Common;
	using Ecng.Trading.Algo.Strategies;
	using Ecng.Xaml;

	class StrategyNode : Disposable
	{
		private readonly StrategiesMonitorWindow _parent;

		public StrategyNode(Strategy strategy, StrategiesMonitorWindow parent)
		{
			if (strategy == null)
				throw new ArgumentNullException("strategy");

			if (parent == null)
				throw new ArgumentNullException("parent");

			this.Strategy = strategy;

			_parent = parent;

			this.ChildNodes = new ThreadSafeObservableCollection<StrategyNode>();
			this.ChildNodes.CollectionChanged += OnChildNodesCollectionChanged;

			foreach (var s in this.Strategy.ChildStrategies)
				OnChildStrategiesAdded(s);

			this.Strategy.ChildStrategies.Added += OnChildStrategiesAdded;
			this.Strategy.ChildStrategies.Removed += OnChildStrategiesRemoved;
			this.Strategy.ChildStrategies.Cleared += OnChildStrategiesCleared;
		}

		private void OnChildStrategiesCleared()
		{
			_parent.GuiSync(() =>
			{
				this.ChildNodes.ForEach(n => _parent.RemoveLogControl(n));
				this.ChildNodes.Clear();
			});
		}

		private void OnChildStrategiesRemoved(Strategy s)
		{
			_parent.GuiSync(() =>
			{
				var node = this.ChildNodes.First(n => n.Strategy == s);
				_parent.RemoveLogControl(node);
				this.ChildNodes.Remove(node);
			});
		}

		private void OnChildStrategiesAdded(Strategy s)
		{
			_parent.GuiSync(() =>
			{
				var node = new StrategyNode(s, _parent);
				_parent.AddLogControl(node);
				this.ChildNodes.Add(node);
			});
		}

		private void OnChildNodesCollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
		{
			if (e.NewItems != null)
			{
				foreach (StrategyNode node in e.NewItems)
					node.ParentNode = this;
			}
		}

		public Strategy Strategy { get; private set; }

		public StrategyNode ParentNode { get; private set; }

		public ObservableCollection<StrategyNode> ChildNodes { get; private set; }

		protected override void DisposeManaged()
		{
			this.ChildNodes.CollectionChanged -= OnChildNodesCollectionChanged;

			this.Strategy.ChildStrategies.Added -= OnChildStrategiesAdded;
			this.Strategy.ChildStrategies.Removed -= OnChildStrategiesRemoved;
			this.Strategy.ChildStrategies.Cleared -= OnChildStrategiesCleared;

			base.DisposeManaged();
		}
	}
}